'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var classNames = function classNames(classHash) {
  return Object.keys(classHash).reduce(function (classString, key) {
    return classHash[key] ? classString + ' ' + key : classString;
  }, '');
};

var FlexChild = function FlexChild(_ref) {
  var className = _ref.className,
      children = _ref.children,
      _grow = _ref._grow,
      _reset = _ref._reset,
      _scroll = _ref._scroll,
      _inlineSizes = _ref._inlineSizes;

  var inlineSizeClasses = {};
  _inlineSizes.forEach(function (size) {
    inlineSizeClasses['flex-child--' + size] = true;
  });

  var childClasses = classNames(Object.assign({}, {
    'flex-child--grow': _grow,
    'flex-child--reset': _reset,
    'flex-child--scroll': _scroll
  }, inlineSizeClasses));

  var renderChildWithClassName = function renderChildWithClassName() {
    return className ? React.createElement(
      'div',
      { className: className },
      children
    ) : children;
  };

  return flexReset || flexScroll ? React.createElement(
    'div',
    { className: childClasses },
    React.createElement(
      'div',
      null,
      renderChildWithClassName()
    )
  ) : React.createElement(
    'div',
    { className: childClasses },
    renderChildWithClassName()
  );
};

FlexChild.propTypes = {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  _grow: PropTypes.bool,
  _reset: PropTypes.bool,
  _scroll: PropTypes.bool,
  _inlineSizes: PropTypes.array
};

FlexChild.defaultProps = {
  className: null,
  _grow: false,
  _reset: false,
  _scroll: false,
  _inlineSizes: []
};

var Flex = function (_React$PureComponent) {
  inherits(Flex, _React$PureComponent);

  function Flex() {
    classCallCheck(this, Flex);
    return possibleConstructorReturn(this, (Flex.__proto__ || Object.getPrototypeOf(Flex)).apply(this, arguments));
  }

  createClass(Flex, [{
    key: 'render',
    value: function render() {
      var flexClasses = classNames({
        'flex': true,
        'flex--inline': this.props.inline,
        'flex--wrap': this.props.wrap,
        'flex--items-center': this.props.itemsCenter,
        'flex--full-height': this.props.fullHeight,
        'flex--container': this.props.container,
        'flex--gutters-half': this.props.guttersHalf,
        'flex--gutters': this.props.gutters,
        'flex--gutters-2x': this.props.gutters2x,
        'flex--gutters-3x': this.props.gutters3x,
        'flex--gutters-4x': this.props.gutters4x,
        'flex--gutters-5x': this.props.gutters5x
      });

      var flexChildren = React.Children.map(this.props.children, function (child) {
        // Wrap any children in a div to prevent potential css flex layout overrides.
        if (child) {
          var flexChildProps = Object.keys(FlexChild.defaultProps).reduce(function (props, key) {
            if (key !== 'className') {
              props[key] = child.props[key];
            }
            return props;
          }, {});

          var childPropsWithoutFlexProps = Object.assign({}, child.props);

          Object.keys(flexChildProps).forEach(function (key) {
            delete childPropsWithoutFlexProps[key];
          });

          var wrapped = React.createElement('div', flexChildProps, Object.assign({}, child, {
            props: childPropsWithoutFlexProps
          }));

          return React.createElement(FlexChild, wrapped.props);
        }
        return child ? React.createElement(FlexChild, child.props) : null;
      });

      return this.props.className ? React.createElement(
        'div',
        { className: this.props.className },
        React.createElement(
          'div',
          { className: flexClasses },
          flexChildren
        )
      ) : React.createElement(
        'div',
        { className: flexClasses },
        flexChildren
      );
    }
  }]);
  return Flex;
}(React.PureComponent);

Flex.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  className: PropTypes.string,
  inline: PropTypes.bool,
  wrap: PropTypes.bool,
  itemsCenter: PropTypes.bool,
  fullHeight: PropTypes.bool,
  container: PropTypes.bool,
  gutters: PropTypes.bool,
  guttersHalf: PropTypes.bool,
  gutters2x: PropTypes.bool,
  gutters3x: PropTypes.bool,
  gutters4x: PropTypes.bool,
  gutters5x: PropTypes.bool
};

Flex.defaultProps = {
  className: null,
  inline: false,
  wrap: false,
  itemsCenter: false,
  fullHeight: false,
  container: false,
  gutters: false,
  guttersHalf: false,
  gutters2x: false,
  gutters3x: false,
  gutters4x: false,
  gutters5x: false
};

exports.Flex = Flex;
